
#include "Client.hh"

#include "Service.hh"
#include "umlrtcommsportrole.hh"
#include "umlrtmessage.hh"
#include "umlrtsignal.hh"
#include "umlrtslot.hh"
#include "umlrttimerprotocol.hh"
#include <cstddef>
#include "umlrtcapsuleclass.hh"
#include "umlrtframeservice.hh"
class UMLRTRtsInterface;
struct UMLRTCommsPort;

Capsule_Client::Capsule_Client( const UMLRTCapsuleClass * cd, UMLRTSlot * st, const UMLRTCommsPort * * border, const UMLRTCommsPort * * internal, bool isStat )
: UMLRTCapsule( NULL, cd, st, border, internal, isStat )
, service( internalPorts[internalport_service] )
, timing( internalPorts[internalport_timing] )
, currentState( SPECIAL_INTERNAL_STATE_UNVISITED )
{
    stateNames[Done] = "Done";
    stateNames[Ready] = "Ready";
    stateNames[TimedOut] = "TimedOut";
    stateNames[WaitingForServiceToBeReady] = "WaitingForServiceToBeReady";
    stateNames[SPECIAL_INTERNAL_STATE_UNVISITED] = "<uninitialized>";
}







void Capsule_Client::bindPort( bool isBorder, int portId, int index )
{
}

void Capsule_Client::unbindPort( bool isBorder, int portId, int index )
{
}

void Capsule_Client::inject( const UMLRTMessage & message )
{
    msg = &message;
    switch( currentState )
    {
    case WaitingForServiceToBeReady:
        currentState = state_____WaitingForServiceToBeReady( &message );
        break;
    case TimedOut:
        currentState = state_____TimedOut( &message );
        break;
    case Ready:
        currentState = state_____Ready( &message );
        break;
    case Done:
        currentState = state_____Done( &message );
        break;
    default:
        break;
    }
}

void Capsule_Client::initialize( const UMLRTMessage & message )
{
    msg = &message;
    actionchain_____Initial( &message );
    currentState = WaitingForServiceToBeReady;
}

const char * Capsule_Client::getCurrentStateString() const
{
    return stateNames[currentState];
}




void Capsule_Client::entryaction_____Done( const UMLRTMessage * msg )
{
    #define rtdata ( (void *)msg->getParam( 0 ) )
    /* UMLRTGEN-USERREGION-BEGIN platform:/resource/SPPInheritance/SPPInheritance.uml RootElement::Client::Done entry  */
    log.log("[Client](Done) response received");
    exit(0);
    /* UMLRTGEN-USERREGION-END */
    #undef rtdata
}

void Capsule_Client::entryaction_____Ready( const UMLRTMessage * msg )
{
    #define rtdata ( (void *)msg->getParam( 0 ) )
    /* UMLRTGEN-USERREGION-BEGIN platform:/resource/SPPInheritance/SPPInheritance.uml RootElement::Client::Ready entry  */
    log.log("[Client](Ready) service ready; sending request");
    timing.informIn(UMLRTTimespec(2,0));
    service.request().send();
    log.log("[Client](Ready) request sent; waiting for response");
    /* UMLRTGEN-USERREGION-END */
    #undef rtdata
}

void Capsule_Client::entryaction_____TimedOut( const UMLRTMessage * msg )
{
    #define rtdata ( (void *)msg->getParam( 0 ) )
    /* UMLRTGEN-USERREGION-BEGIN platform:/resource/SPPInheritance/SPPInheritance.uml RootElement::Client::TimedOut entry  */
    log.log("[Client](TimedOut)");
    /* UMLRTGEN-USERREGION-END */
    #undef rtdata
}

void Capsule_Client::entryaction_____WaitingForServiceToBeReady( const UMLRTMessage * msg )
{
    #define rtdata ( (void *)msg->getParam( 0 ) )
    /* UMLRTGEN-USERREGION-BEGIN platform:/resource/SPPInheritance/SPPInheritance.uml RootElement::Client::WaitingForServiceToBeReady entry  */
    log.log("[Client](WaitingForServiceToBeReady) setting timer");
    timing.informIn(UMLRTTimespec(2,0));
    /* UMLRTGEN-USERREGION-END */
    #undef rtdata
}

void Capsule_Client::exitaction_____Ready( const UMLRTMessage * msg )
{
    #define rtdata ( (void *)msg->getParam( 0 ) )
    /* UMLRTGEN-USERREGION-BEGIN platform:/resource/SPPInheritance/SPPInheritance.uml RootElement::Client::Ready exit  */
    /* UMLRTGEN-USERREGION-END */
    #undef rtdata
}

void Capsule_Client::exitaction_____WaitingForServiceToBeReady( const UMLRTMessage * msg )
{
    #define rtdata ( (void *)msg->getParam( 0 ) )
    /* UMLRTGEN-USERREGION-BEGIN platform:/resource/SPPInheritance/SPPInheritance.uml RootElement::Client::WaitingForServiceToBeReady exit  */
    /* UMLRTGEN-USERREGION-END */
    #undef rtdata
}

void Capsule_Client::actionchain_____Initial( const UMLRTMessage * msg )
{
    entryaction_____WaitingForServiceToBeReady( msg );
}

void Capsule_Client::actionchain_____TransitionAction2( const UMLRTMessage * msg )
{
    exitaction_____WaitingForServiceToBeReady( msg );
    entryaction_____Ready( msg );
}

void Capsule_Client::actionchain_____TransitionAction3( const UMLRTMessage * msg )
{
    exitaction_____Ready( msg );
    entryaction_____Done( msg );
}

void Capsule_Client::actionchain_____timeout1( const UMLRTMessage * msg )
{
    exitaction_____WaitingForServiceToBeReady( msg );
    entryaction_____TimedOut( msg );
}

void Capsule_Client::actionchain_____timeout2( const UMLRTMessage * msg )
{
    exitaction_____Ready( msg );
    entryaction_____TimedOut( msg );
}

Capsule_Client::State Capsule_Client::state_____Done( const UMLRTMessage * msg )
{
    switch( msg->destPort->role()->id )
    {
    default:
        this->unexpectedMessage();
        break;
    }
    return currentState;
}

Capsule_Client::State Capsule_Client::state_____Ready( const UMLRTMessage * msg )
{
    switch( msg->destPort->role()->id )
    {
    case port_service:
        switch( msg->getSignalId() )
        {
        case Service::signal_response:
            actionchain_____TransitionAction3( msg );
            return Done;
        default:
            this->unexpectedMessage();
            break;
        }
        return currentState;
    case port_timing:
        switch( msg->getSignalId() )
        {
        case UMLRTTimerProtocol::signal_timeout:
            actionchain_____timeout2( msg );
            return TimedOut;
        default:
            this->unexpectedMessage();
            break;
        }
        return currentState;
    default:
        this->unexpectedMessage();
        break;
    }
    return currentState;
}

Capsule_Client::State Capsule_Client::state_____TimedOut( const UMLRTMessage * msg )
{
    switch( msg->destPort->role()->id )
    {
    default:
        this->unexpectedMessage();
        break;
    }
    return currentState;
}

Capsule_Client::State Capsule_Client::state_____WaitingForServiceToBeReady( const UMLRTMessage * msg )
{
    switch( msg->destPort->role()->id )
    {
    case port_timing:
        switch( msg->getSignalId() )
        {
        case UMLRTTimerProtocol::signal_timeout:
            actionchain_____timeout1( msg );
            return TimedOut;
        default:
            this->unexpectedMessage();
            break;
        }
        return currentState;
    case port_service:
        switch( msg->getSignalId() )
        {
        case UMLRTSignal::rtBound:
            actionchain_____TransitionAction2( msg );
            return Ready;
        default:
            this->unexpectedMessage();
            break;
        }
        return currentState;
    default:
        this->unexpectedMessage();
        break;
    }
    return currentState;
}


static const UMLRTCommsPortRole portroles_internal[] = 
{
    {
        Capsule_Client::port_service,
        "Service",
        "service",
        "",
        1,
        true,
        true,
        false,
        true,
        true,
        false,
        false
    },
    {
        Capsule_Client::port_timing,
        "Timing",
        "timing",
        "",
        1,
        true,
        false,
        false,
        false,
        true,
        false,
        false
    },
    {
        Capsule_Client::port_log,
        "Log",
        "log",
        "",
        1,
        true,
        false,
        false,
        false,
        true,
        false,
        false
    }
};

static void instantiate_Client( const UMLRTRtsInterface * rts, UMLRTSlot * slot, const UMLRTCommsPort * * borderPorts )
{
    const UMLRTCommsPort * * internalPorts = UMLRTFrameService::createInternalPorts( slot, &Client );
    slot->capsule = new Capsule_Client( &Client, slot, borderPorts, internalPorts, false );
}

const UMLRTCapsuleClass Client = 
{
    "Client",
    NULL,
    instantiate_Client,
    0,
    NULL,
    0,
    NULL,
    3,
    portroles_internal
};

